@@ -51,6 +51,8 @@ pub enum Sdl2Subsystem
 #[derive(Debug, Fail)]
 pub enum InitError
 {
+    #[fail(display = "{}", _0)]
+    KeeshondInitError(String),
     #[fail(display = "SDL {:?} subsystem: {}", _0, _1)]
     Sdl2InitError(Sdl2Subsystem, String),
     #[fail(display = "{}", _0)]
@@ -258,6 +260,17 @@ pub enum FrameInterpolationCap
     None
 }
 
+/// Whether to space out display frames for smoother motion when the framerate is higher than the target rate and there is significant logic processing.
+#[derive(Copy, Clone, Debug, PartialEq)]
+pub enum FrameInterpolationPacing
+{
+    /// Regulate the timing of display frames so that they are more evenly spaced apart, albeit
+    ///  lowering the framerate (except when it falls below the target framerate).
+    PreferSmoothMotion,
+    /// Don't regulate the timing of display frames, increasing framerate.
+    PreferHighFramerate
+}
+
 /// Properties and metadata for this game
 #[derive(Clone)]
 pub struct GameInfo
@@ -287,6 +300,9 @@ pub struct GameInfo
     /// Optional framerate cap for frame interpolation mode, useful for keeping the
     ///  display framerate below the maximum supported VRR rate.
     pub frame_interpolation_cap : FrameInterpolationCap,
+    /// Whether to space out display frames for smoother motion when the framerate is higher than
+    ///  the target rate and there is significant logic processing.
+    pub frame_interpolation_pacing : FrameInterpolationPacing,
     /// Whether to display the mouse cursor when it is over the window.
     pub cursor_visible : bool,
     /// Whether unused resources should be automatically unloaded between scenes.
@@ -331,6 +347,7 @@ impl Default for GameInfo
             target_framerate : 60.0,
             frame_interpolation : FrameInterpolationMode::Off,
             frame_interpolation_cap : FrameInterpolationCap::Automatic,
+            frame_interpolation_pacing : FrameInterpolationPacing::PreferSmoothMotion,
             log_stats : true,
             log_packages : Vec::new()
         }
@@ -486,6 +503,7 @@ pub struct GameControl
     target_framerate : f64,
     frame_interpolation : FrameInterpolationMode,
     interpolation_cap : FrameInterpolationCap,
+    interpolation_pacing : FrameInterpolationPacing,
     low_energy_app : bool,
     low_energy_timer : u32,
     auto_unload : bool,
@@ -591,6 +609,7 @@ impl GameControl
             target_framerate : gameinfo.target_framerate,
             frame_interpolation : gameinfo.frame_interpolation,
             interpolation_cap : gameinfo.frame_interpolation_cap,
+            interpolation_pacing : gameinfo.frame_interpolation_pacing,
             low_energy_app : gameinfo.low_energy_app,
             low_energy_timer : 0,
             auto_unload : gameinfo.auto_unload,
@@ -763,6 +782,18 @@ impl GameControl
         self.interpolation_cap = cap;
     }
 
+    /// Returns the framerate pacing mode for when frame interpolation is enabled.
+    pub fn frame_interpolation_pacing(&self) -> FrameInterpolationPacing
+    {
+        self.interpolation_pacing
+    }
+
+    /// Sets the framerate pacing mode for when frame interpolation is enabled.
+    pub fn set_frame_interpolation_pacing(&mut self, pacing : FrameInterpolationPacing)
+    {
+        self.interpolation_pacing = pacing;
+    }
+
     /// Returns true if frame interpolation is currently active. For the automatic mode, returns
     ///  true if the refresh rate of the window's current monitor does not match with the target
     ///  logic framerate.
@@ -826,12 +857,14 @@ struct GameloopTimeState
     target_frametime : Duration,
     cap_frametime : Duration,
     frame_start_jiffies : Duration,
+    first_draw_jiffies : Duration,
     last_display : Instant,
     #[cfg(feature = "imgui_feature")]
     last_imgui : Instant,
     lag_this_frame : u32,
     lag_this_second : u32,
     thought_yet : bool,
+    drawn_thought_yet : bool,
     lag_recovery : bool,
     refresh_rate : i32,
     think_timer : GameloopStopwatch,
@@ -856,12 +889,14 @@ impl GameloopTimeState
             target_frametime : Duration::new(0, 0),
             cap_frametime : Duration::new(0, 0),
             frame_start_jiffies : Duration::new(0, 0),
+            first_draw_jiffies : Duration::new(0, 0),
             last_display : Instant::now(),
             #[cfg(feature = "imgui_feature")]
             last_imgui : Instant::now(),
             lag_this_frame : 0,
             lag_this_second : 0,
             thought_yet : false,
+            drawn_thought_yet : false,
             lag_recovery : false,
             refresh_rate : 0,
             think_timer : GameloopStopwatch::new(),
@@ -1037,11 +1072,17 @@ impl Gameloop
     /// Creates the main game context. Will return an error instead of panicking if startup fails for any reason.
     pub fn try_new(gameinfo : GameInfo) -> Result<Gameloop, InitError>
     {
+        let gameloop_init_start = Instant::now();
+
+        // Messagebox panic handler
+        #[cfg(feature = "graphical_panic")]
+        std::panic::set_hook(Box::new(panic_hook));
+
         let mut gameinfo = gameinfo.clone();
         
         if gameinfo.package_name.is_empty()
         {
-            panic!("Package name cannot be empty.");
+            return Err(InitError::KeeshondInitError(String::from("Package name cannot be empty.")));
         }
 
         let console_history = Arc::new(Mutex::new(ConsoleHistory::new()));
@@ -1067,10 +1108,6 @@ impl Gameloop
             }
         }
         
-        // Messagebox panic handler
-        #[cfg(feature = "graphical_panic")]
-        std::panic::set_hook(Box::new(panic_hook));
-        
         apply_config(&mut gameinfo);
         
         print_log_section("KEESHOND Game Engine");
@@ -1093,8 +1130,10 @@ impl Gameloop
         
         control.renderer_mut().set_base_size(gameinfo.base_width as f32, gameinfo.base_height as f32);
         control.renderer_mut().set_window_title(&gameinfo.friendly_name);
-        
-        info!("Gameloop constructed");
+
+        let gameloop_init_time = (Instant::now() - gameloop_init_start).as_millis();
+
+        info!("Initialized in {} ms", gameloop_init_time);
         
         Ok(Gameloop
         {
@@ -1130,7 +1169,7 @@ impl Gameloop
                     * AUTOMATIC_LERP_CAP_MULTIPLIER)
             }
             FrameInterpolationCap::Manual(value) => { 1000.0 / value }
-            FrameInterpolationCap::None => { 1.0 }
+            FrameInterpolationCap::None => { 0.0 }
         };
         self.time.target_frametime = Duration::from_nanos((target_ms * 1_000_000.0) as u64);
         self.time.cap_frametime = Duration::from_nanos((cap_ms * 1_000_000.0) as u64);
@@ -1166,6 +1205,7 @@ impl Gameloop
             
             self.time.think_timer.tic();
             self.time.thought_yet = true;
+            self.time.drawn_thought_yet = false;
             self.control.low_energy_timer += 1;
 
             let max_think_jiffies = self.time.target_frametime;
@@ -1227,6 +1267,12 @@ impl Gameloop
         }
         
         self.time.draw_timer.end_frame_timing();
+
+        if !self.time.drawn_thought_yet
+        {
+            self.time.first_draw_jiffies = self.time.draw_timer.jiffies_this_frame;
+            self.time.drawn_thought_yet = true;
+        }
     }
 
     fn scene_draw(&mut self)
@@ -1312,14 +1358,28 @@ impl Gameloop
         {
             self.time.sleep(self.time.target_frametime, low_energy);
         }
-        else if self.control.frame_interpolation_cap() != FrameInterpolationCap::None
-        {
-            self.time.interpolate_sleep(self.time.cap_frametime, low_energy);
-            self.time.update_jiffies();
-        }
         else
         {
-            self.time.last_display = Instant::now();
+            let mut cap_frametime = self.time.cap_frametime;
+
+            if self.control.interpolation_pacing != FrameInterpolationPacing::PreferHighFramerate
+            {
+                let pacing_target = self.time.think_timer.jiffies_this_frame
+                    + self.time.first_draw_jiffies;
+
+                cap_frametime = self.time.cap_frametime.max(pacing_target)
+                    .min(self.time.target_frametime);
+            }
+
+            if !cap_frametime.is_zero()
+            {
+                self.time.interpolate_sleep(cap_frametime, low_energy);
+            }
+            else
+            {
+                self.time.last_display = Instant::now();
+            }
+
             self.time.update_jiffies();
         }
         
@@ -1573,8 +1633,13 @@ impl Gameloop
             std::mem::swap(&mut self.current_scene, &mut self.control.next_scene);
             self.control.next_scene = Box::new(Scene::<PlaceholderScene>::new());
 
+            let scene_init_start = Instant::now();
+
             self.current_scene.start(&mut self.control);
 
+            let scene_init_time = (Instant::now() - scene_init_start).as_millis();
+            info!("Scene started in {} ms", scene_init_time);
+
             // Reset jiffy queue
             self.time.flush();
             self.control.next_scene_waiting = false;
