@@ -31,7 +31,7 @@ use std::str::FromStr;
 use crate::datapack::DataMultistore;
 use crate::gameloop::GameControl;
 use crate::renderer::{DrawControl, DrawTransform};
-use crate::scene::iter::{ComponentAllIter, ComponentAllIterMut, ComponentFlagsIter, ComponentFilterIterMut, ComponentIter, ComponentIterMut, ComponentFilterIter, ComponentFlagsIterMut};
+use crate::scene::iter::{ComponentAllIter, ComponentAllIterMut, ComponentIter, ComponentIterMut, ComponentFilterIter, ComponentFilterIterMut};
 
 #[cfg(feature = "imgui_feature")]
 use std::marker::PhantomData;
@@ -39,6 +39,7 @@ use std::marker::PhantomData;
 type ComponentTypeMax = U1024;
 
 const MAX_RESERVE_CAPACITY_HINT : usize = 2097152;
+const CHUNKED_VEC_CHUNK_SIZE : usize = 1024; // Size of a 4kB memory page
 
 /// Helper macro to define an enum of different spawnable objects. Basic format:
 ///
@@ -721,6 +722,138 @@ struct ComponentListEntry<C : Component + 'static>
     freeze_flags : FreezeFlags
 }
 
+trait EntityMapping
+{
+    fn with_capacity(capacity_hint : usize) -> Self where Self : Sized;
+    fn get(&self, entity_id : usize) -> Option<usize>;
+    fn contains(&self, entity_id : usize) -> bool;
+    fn add(&mut self, entity_id : usize, index : usize);
+    fn set(&mut self, entity_id : usize, index : usize);
+    fn remove(&mut self, entity_id : usize);
+    fn reserve(&mut self, additional : usize);
+    fn shrink_to_fit(&mut self);
+}
+
+#[derive(Debug)]
+struct ChunkedVecEntityMapping
+{
+    lists : Vec<Option<Box<[u32; CHUNKED_VEC_CHUNK_SIZE]>>>,
+    counts : Vec<usize>
+}
+
+impl EntityMapping for ChunkedVecEntityMapping
+{
+    fn with_capacity(capacity_hint : usize) -> Self where Self : Sized
+    {
+        ChunkedVecEntityMapping
+        {
+            lists : Vec::with_capacity(capacity_hint / CHUNKED_VEC_CHUNK_SIZE),
+            counts : Vec::with_capacity(capacity_hint / CHUNKED_VEC_CHUNK_SIZE)
+        }
+    }
+
+    fn get(&self, entity_id : usize) -> Option<usize>
+    {
+        let first_index = entity_id / CHUNKED_VEC_CHUNK_SIZE;
+
+        if first_index < self.lists.len()
+        {
+            if let Some(list) = &self.lists[first_index]
+            {
+                let value = list[entity_id % CHUNKED_VEC_CHUNK_SIZE];
+
+                if value < u32::MAX
+                {
+                    return Some(value as usize);
+                }
+            }
+        }
+
+        None
+    }
+
+    fn contains(&self, entity_id : usize) -> bool
+    {
+        let first_index = entity_id / CHUNKED_VEC_CHUNK_SIZE;
+
+        if first_index < self.lists.len()
+        {
+            if let Some(list) = &self.lists[first_index]
+            {
+                let value = list[entity_id % CHUNKED_VEC_CHUNK_SIZE];
+
+                return value < u32::MAX;
+            }
+        }
+
+        false
+    }
+
+    fn add(&mut self, entity_id : usize, index : usize)
+    {
+        // This is already checked in SceneControl::get_next_entity(), but just in case...
+        debug_assert!((entity_id as u32) < u32::MAX, "Too many entities");
+
+        let first_index = entity_id / CHUNKED_VEC_CHUNK_SIZE;
+
+        while first_index >= self.lists.len()
+        {
+            // TODO: Make faster?
+            self.lists.push(None);
+            self.counts.push(0);
+        }
+
+        if self.lists[first_index].is_none()
+        {
+            self.lists[first_index] = Some(Box::new([u32::MAX; CHUNKED_VEC_CHUNK_SIZE]));
+        }
+
+        let list = &mut self.lists[first_index].as_mut().unwrap();
+
+        list[entity_id % CHUNKED_VEC_CHUNK_SIZE] = index as u32;
+
+        self.counts[first_index] += 1;
+    }
+
+    fn set(&mut self, entity_id : usize, index : usize)
+    {
+        let first_index = entity_id / CHUNKED_VEC_CHUNK_SIZE;
+        let list = &mut self.lists[first_index].as_mut().unwrap();
+
+        list[entity_id % CHUNKED_VEC_CHUNK_SIZE] = index as u32;
+    }
+
+    fn remove(&mut self, entity_id : usize)
+    {
+        let first_index = entity_id / CHUNKED_VEC_CHUNK_SIZE;
+
+        if first_index < self.lists.len()
+        {
+            if let Some(list) = &mut self.lists[first_index]
+            {
+                list[entity_id % CHUNKED_VEC_CHUNK_SIZE] = u32::MAX;
+
+                self.counts[first_index] -= 1;
+
+                if self.counts[first_index] == 0
+                {
+                    self.lists[first_index] = None;
+                }
+            }
+        }
+    }
+
+    fn reserve(&mut self, _additional : usize)
+    {
+        // No-op
+    }
+
+    fn shrink_to_fit(&mut self)
+    {
+        // No-op
+    }
+}
+
 /// Stores all instances of a [Component] for a given type, indexed based on the [Entities](Entity)
 /// they belong to.
 #[derive(Debug)]
@@ -728,7 +861,7 @@ pub struct ComponentStore<C : Component + 'static>
 {
     component_list : Vec<ComponentListEntry<C>>,
     removed_components : BitSet<u32>,
-    entity_mapping : Vec<Option<usize>>,
+    entity_mapping : ChunkedVecEntityMapping,
     singleton_item : Option<C>,
     need_purge : bool,
     ordered_removal : bool
@@ -742,24 +875,18 @@ impl<C : Component + 'static> ComponentStore<C>
         {
             component_list : Vec::with_capacity(capacity_hint),
             removed_components : BitSet::new(),
-            entity_mapping : Vec::with_capacity(capacity_hint),
+            entity_mapping : ChunkedVecEntityMapping::with_capacity(capacity_hint),
             singleton_item : None,
             need_purge : false,
             ordered_removal
         }
     }
-    
+
     fn add_entity(&mut self, entity : &Entity, component : C) -> bool
     {
-        while entity.id >= self.entity_mapping.len()
+        if !self.entity_mapping.contains(entity.id) || self.removed_components.contains(entity.id)
         {
-            // TODO: Make faster?
-            self.entity_mapping.push(None);
-        }
-
-        if self.entity_mapping[entity.id].is_none() || self.removed_components.contains(entity.id)
-        {
-            self.entity_mapping[entity.id] = Some(self.component_list.len());
+            self.entity_mapping.add(entity.id, self.component_list.len());
             self.component_list.push(ComponentListEntry
             {
                 component, entity : entity.clone(), freeze_flags : FreezeFlags::empty()
@@ -777,12 +904,7 @@ impl<C : Component + 'static> ComponentStore<C>
     #[inline]
     fn get_nth_index(&self, entity : &Entity) -> Option<usize>
     {
-        if entity.id >= self.entity_mapping.len()
-        {
-            return None;
-        }
-
-        if let Some(index) = self.entity_mapping[entity.id]
+        if let Some(index) = self.entity_mapping.get(entity.id)
         {
             let entry = &self.component_list[index];
 
@@ -791,12 +913,12 @@ impl<C : Component + 'static> ComponentStore<C>
                 return None;
             }
 
-            return Some(index);
+            return Some(index as usize);
         }
 
         None
     }
-    
+
     /// Returns an immutable reference to the [Component] for the given [Entity], or [None]
     /// if it does not exist.
     pub fn get_entity<'a>(&'a self, entity : &Entity) -> Option<C::StoreRef> where C : ComponentRef<'a>
@@ -810,7 +932,7 @@ impl<C : Component + 'static> ComponentStore<C>
             }
         }
     }
-    
+
     /// Returns a mutable reference to the [Component] for the given [Entity], or [None]
     /// if it does not exist.
     pub fn get_entity_mut<'a>(&'a mut self, entity : &Entity) -> Option<C::StoreRefMut> where C : ComponentRefMut<'a>
@@ -881,13 +1003,13 @@ impl<C : Component + 'static> ComponentStore<C>
     {
         self.get_nth_mut(0)
     }
-    
+
     /// Returns the number of [Entities](Entity) with this [Component]
     pub fn count(&self) -> usize
     {
         self.component_list.len()
     }
-    
+
     /// Returns true if there is a [Component] for the given [Entity]
     pub fn contains(&self, entity : &Entity) -> bool
     {
@@ -954,7 +1076,7 @@ impl<C : Component + 'static> ComponentStore<C>
             }
         }
     }
-    
+
     pub fn sort<F : FnMut(&C, &C) -> Ordering>(&mut self, mut compare : F)
     {
         self.component_list.sort_unstable_by(|a, b|
@@ -965,12 +1087,12 @@ impl<C : Component + 'static> ComponentStore<C>
                 other => { other }
             }
         });
-        
+
         for i in 0..self.component_list.len()
         {
             let entry = &self.component_list[i];
-            
-            self.entity_mapping[entry.entity.id] = Some(i);
+
+            self.entity_mapping.set(entry.entity.id, i);
         }
     }
 
@@ -1018,42 +1140,28 @@ impl<C : Component + 'static> ComponentStore<C>
         ComponentFilterIterMut::<C>::new(self, freeze_flags)
     }
 
-    /// Returns a [ComponentFilterIter] that iterates through all [Components](Component)
-    ///  in the store along with their [FreezeFlags] for custom filtering.
-    pub fn iter_flags<'a>(&'a self) -> ComponentFlagsIter<C> where C : ComponentRef<'a>
-    {
-        ComponentFlagsIter::<C>::new(self)
-    }
-
-    /// Returns a [ComponentFlagsIterMut] that iterates through all [Components](Component)
-    ///  in the store along with their [FreezeFlags] for custom filtering.
-    pub fn iter_flags_mut<'a>(&'a mut self) -> ComponentFlagsIterMut<C> where C : ComponentRefMut<'a>
-    {
-        ComponentFlagsIterMut::<C>::new(self)
-    }
-    
     pub fn singleton(&self) -> Option<&C>
     {
         self.singleton_item.as_ref()
     }
-    
+
     pub fn singleton_mut(&mut self) -> &mut C where C : Default
     {
         if self.singleton_item.is_none()
         {
             self.singleton_item = Some(C::default());
         }
-        
+
         self.singleton_item.as_mut().unwrap()
     }
-    
+
     pub fn singleton_mut_or<F : FnMut() -> C>(&mut self, mut func : F) -> &mut C
     {
         if self.singleton_item.is_none()
         {
             self.singleton_item = Some(func());
         }
-        
+
         self.singleton_item.as_mut().unwrap()
     }
 
@@ -1074,12 +1182,7 @@ impl<C : Component + 'static> ComponentStoreBase for ComponentStore<C>
 {
     fn preview_remove_entity(&mut self, entity : &Entity) -> bool
     {
-        if entity.id >= self.entity_mapping.len()
-        {
-            return false;
-        }
-
-        if let Some(index) = self.entity_mapping[entity.id]
+        if let Some(index) = self.entity_mapping.get(entity.id)
         {
             let entry = &mut self.component_list[index];
 
@@ -1096,11 +1199,11 @@ impl<C : Component + 'static> ComponentStoreBase for ComponentStore<C>
 
     fn do_remove_entity(&mut self, entity : &Entity)
     {
-        if let Some(index) = self.entity_mapping[entity.id]
+        if let Some(index) = self.entity_mapping.get(entity.id)
         {
             if self.ordered_removal
             {
-                self.entity_mapping[entity.id] = None;
+                self.entity_mapping.remove(entity.id);
                 self.removed_components.insert(entity.id);
                 self.need_purge = true;
             }
@@ -1109,11 +1212,11 @@ impl<C : Component + 'static> ComponentStoreBase for ComponentStore<C>
                 if !self.component_list.is_empty()
                 {
                     let entry = &self.component_list[self.component_list.len() - 1];
-                    self.entity_mapping[entry.entity.id] = self.entity_mapping[entity.id];
+                    self.entity_mapping.set(entry.entity.id, index);
                 }
 
                 self.component_list.swap_remove(index);
-                self.entity_mapping[entity.id] = None;
+                self.entity_mapping.remove(entity.id);
             }
         }
         else
@@ -1141,7 +1244,7 @@ impl<C : Component + 'static> ComponentStoreBase for ComponentStore<C>
         {
             let entry = &self.component_list[i];
 
-            self.entity_mapping[entry.entity.id] = Some(i);
+            self.entity_mapping.set(entry.entity.id, i);
         }
 
         self.removed_components.clear();
@@ -1150,9 +1253,9 @@ impl<C : Component + 'static> ComponentStoreBase for ComponentStore<C>
 
     fn do_set_frozen(&mut self, entity : &Entity, freeze_flags : FreezeFlags, freeze : bool)
     {
-        if let Some(index) = self.entity_mapping[entity.id]
+        if let Some(index) = self.entity_mapping.get(entity.id)
         {
-            self.component_list[index].freeze_flags.set(freeze_flags, freeze);
+            self.component_list[index as usize].freeze_flags.set(freeze_flags, freeze);
         }
     }
 }
@@ -1273,7 +1376,7 @@ impl ComponentControl
             locked : false
         }
     }
-    
+
     fn register_component_type<C : Component + 'static>(&self) -> bool
     {
         let wanted_type = TypeId::of::<C>();
@@ -1343,7 +1446,7 @@ impl ComponentControl
             }
         }
     }
-    
+
     pub fn try_store<C : Component + 'static>(&self) -> Result<Ref<ComponentStore<C>>, ComponentControlError>
     {
         let wanted_type = TypeId::of::<C>();
@@ -1359,7 +1462,7 @@ impl ComponentControl
         Ok(Ref::map(self.component_stores[index].try_borrow()?,
                              |b| b.downcast_ref::<ComponentStore<C>>().expect("ComponentStore downcast failed!")))
     }
-    
+
     pub fn try_store_mut<C : Component + 'static>(&self) -> Result<RefMut<ComponentStore<C>>, ComponentControlError>
     {
         if self.locked
@@ -1528,10 +1631,10 @@ impl<T : SceneType + 'static> SceneControl<T>
     pub fn add_entity_later(&mut self) -> Entity
     {
         let entity = self.get_next_entity();
-        
+
         self.deferred_adds.push(entity.clone());
         self.deferred_flag = true;
-        
+
         entity
     }
 
@@ -1549,7 +1652,7 @@ impl<T : SceneType + 'static> SceneControl<T>
     {
         self.spawn_control.deferred_adds.push(entity.clone(), component);
         self.create_component_factory::<C>();
-        
+
         self.deferred_flag = true;
     }
 
@@ -1612,13 +1715,18 @@ impl<T : SceneType + 'static> SceneControl<T>
         self.deferred_callback_adds.push((TypeId::of::<C>(), callbacks));
         self.deferred_flag = true;
     }
-    
+
     fn get_next_entity(&mut self) -> Entity
     {
         let entity;
-        
+
         if self.unused_pool.is_empty()
         {
+            if self.next_entity_id as u32 >= u32::MAX
+            {
+                panic!("Too many entities");
+            }
+
             entity = Entity { id : self.next_entity_id, generation : 1 };
             
             self.next_entity_id += 1;
