@@ -269,13 +269,38 @@ impl ThinkerSystem<NullSceneType> for FooBarFreezeSystem
             bar.runs += 1;
         }
 
-        for (_, baz) in baz_store.iter_filtered_mut(FreezeFlags::REASON_HIDDEN)
+        for (_, baz, _) in baz_store.iter_filtered_mut(FreezeFlags::REASON_HIDDEN)
         {
             baz.runs += 1;
         }
     }
 }
 
+#[derive(Debug)]
+struct FooSystemOrdered {}
+
+impl FooSystemOrdered
+{
+    fn new() -> FooSystemOrdered
+    {
+        FooSystemOrdered {}
+    }
+}
+
+impl ThinkerSystem<NullSceneType> for FooSystemOrdered
+{
+    fn think(&mut self, args : ThinkerArgs<NullSceneType>)
+    {
+        let mut foo_store = args.components.store_mut::<FooComponentOrdered>();
+        let iter = foo_store.iter_mut();
+
+        for (_, foo) in iter
+        {
+            foo.runs += 1;
+        }
+    }
+}
+
 #[derive(Debug)]
 struct FooBarSystemOrdered {}
 
@@ -605,6 +630,58 @@ fn scene_remove_entity_component()
     assert!(store.get_entity(&entity2).is_none());
 }
 
+#[test]
+fn scene_remove_entity_component_ordered()
+{
+    let mut scene = SimpleScene::new();
+    let mut gameloop = GameControl::new_headless(&GameInfo::default()).unwrap();
+
+    let entity1 = scene.add_entity();
+    let entity2 = scene.add_entity();
+
+    assert_eq!(scene.add_component(&entity1, FooComponentOrdered::new()), true);
+    assert_eq!(scene.add_component(&entity2, FooComponentOrdered::new()), true);
+
+    scene.add_thinker_system(Box::new(FooSystemOrdered::new()));
+
+    scene.think(&mut gameloop);
+
+    let store = scene.component_control().store::<FooComponentOrdered>();
+    assert_eq!(store.get_entity(&entity1).unwrap().runs, 1);
+    assert_eq!(store.get_entity(&entity2).unwrap().runs, 1);
+
+    std::mem::drop(store);
+
+    // Prevent adding component where one already exists
+
+    assert_eq!(scene.add_component(&entity1, FooComponentOrdered::new()), false);
+    assert_eq!(scene.remove_component::<FooComponentOrdered>(&entity1), true);
+
+    // Prevent duplicate removal
+
+    assert_eq!(scene.remove_component::<FooComponentOrdered>(&entity1), false);
+
+    let store = scene.component_control().store::<FooComponentOrdered>();
+    assert!(store.get_entity(&entity1).is_none());
+
+    std::mem::drop(store);
+
+    scene.think(&mut gameloop);
+
+    let store = scene.component_control().store::<FooComponentOrdered>();
+    assert_eq!(store.get_entity(&entity2).unwrap().runs, 2);
+
+    std::mem::drop(store);
+
+    assert_eq!(scene.remove_entity(&entity2), true);
+    assert_eq!(scene.remove_entity(&entity2), false);
+
+    scene.think(&mut gameloop);
+
+    let store = scene.component_control().store::<FooComponentOrdered>();
+    assert!(store.get_entity(&entity2).is_none());
+}
+
 #[test]
 fn scene_remove_many()
 {
