@@ -1,2 +1,2 @@
-#[cfg(feature = "renderer")] pub(crate) mod glium_sdl2;
+pub(crate) mod glium_sdl2;
 #[cfg(target_os = "emscripten")] pub(crate) mod emscripten;
